@@ -8,6 +8,7 @@ from .datasets import load_chen_10x
 from .datasets import load_cell_cycle
 from .datasets import load_pal_10x
 from .datasets import load_spirals
+from .datasets import load_test_datasets_random
 from .datasets import load_test_datasets_small
 from .datasets import load_travaglini_10x
 from .datasets import load_zhou_10x
@@ -17,6 +18,7 @@ __all__ = [
     "load_bank",
     "remove_bank",
     "load_spirals",
+    "load_test_datasets_random",
     "load_test_datasets_small",
     "load_cell_cycle",
     "load_chen_10x",
