@@ -12,6 +12,7 @@ from socket import error as SocketError
 from typing import Dict, Optional
 from warnings import warn
 
+from ..utils.file import adapt_path
 from .._logging import logger
 
 CALLBACK = 30
@@ -21,33 +22,33 @@ DATA_PATH = f"{path.dirname(__file__)}/data/"
 
 def check_md5_file(file_path: str, target_hash: str) -> bool:
     # Returns a md5 hash to quality check
-    file_hash = hashlib.md5(open(file_path, "rb").read()).hexdigest()
+    file_hash = hashlib.md5(open(adapt_path(file_path), "rb").read()).hexdigest()
     return file_hash == target_hash
 
 
 def remove_dataset(dataset_name: str) -> None:
     # Removes all files from data/$dataset_name
     logger.log(logging.DEBUG, f"databank_api > Removing dataset {dataset_name}...")
-    if not path.exists(f"{DATA_PATH}{dataset_name}"):
+    if not path.exists(adapt_path(f"{DATA_PATH}{dataset_name}")):
         logger.log(logging.DEBUG, "databank_api > Nothing to do.")
         return
-    for fname in os.listdir(f"{DATA_PATH}{dataset_name}/"):
-        os.remove(f"{DATA_PATH}{dataset_name}/{fname}")
-    os.rmdir(f"{DATA_PATH}{dataset_name}")
+    for fname in os.listdir(adapt_path(f"{DATA_PATH}{dataset_name}/")):
+        os.remove(adapt_path(f"{DATA_PATH}{dataset_name}/{fname}"))
+    os.rmdir(adapt_path(f"{DATA_PATH}{dataset_name}"))
 
 
 def unzip_file(zip_path: str, dataset_name: str) -> None:
     # Unzips $zip_path at data/$dataset_name/
     logger.log(logging.DEBUG, f"databank_api > Unzipping file {zip_path}...")
     with zipfile.ZipFile(zip_path, "r") as fzip:
-        fzip.extractall(f"{DATA_PATH}{dataset_name}/")
-    os.remove(zip_path)
+        fzip.extractall(adapt_path(f"{DATA_PATH}{dataset_name}/"))
+    os.remove(adapt_path(zip_path))
 
 
 def check_files(dataset_name: str) -> bool:
     # Check if all files from a dataset are present
     logger.log(logging.DEBUG, f"databank_api > Checking files from {dataset_name}...")
-    with open(f"{path.dirname(__file__)}/{DATASETS_JSON}", "r") as f:
+    with open(adapt_path(f"{path.dirname(__file__)}/{DATASETS_JSON}"), "r") as f:
         all_datasets = json.load(f)
         dataset: Optional[Dict] = None
         for ds in all_datasets:
@@ -58,7 +59,9 @@ def check_files(dataset_name: str) -> bool:
             warn(f"Dataset {dataset_name} not found in database.")
             return False
         for fname in dataset["files"]:
-            if not path.exists(f"{path.dirname(__file__)}/data/{dataset_name}/{fname}"):
+            if not path.exists(
+                adapt_path(f"{path.dirname(__file__)}/data/{dataset_name}/{fname}")
+            ):
                 return False
     return True
 
@@ -87,10 +90,10 @@ def download_dataset(dataset_name: str) -> str:
     # using data from datasets.json
     # TODO print -> log
 
-    datasets_root = f"{path.dirname(__file__)}/"
+    datasets_root = adapt_path(f"{path.dirname(__file__)}/")
 
     # Ensuring dataset exists
-    f = open(f"{datasets_root}{DATASETS_JSON}", "r")
+    f = open(adapt_path(f"{datasets_root}{DATASETS_JSON}"), "r")
     all_datasets = json.load(f)
     dataset = None
     for ds in all_datasets:
@@ -104,10 +107,10 @@ def download_dataset(dataset_name: str) -> str:
     logger.log(logging.INFO, f"databank_api > Number of files: {dataset['n_files']}")
 
     # Preparing dir structure
-    if not path.exists(f"{datasets_root}data/"):
+    if not path.exists(adapt_path(f"{datasets_root}data/")):
         logger.log(logging.DEBUG, "databank_api > data/ not found, creating it.")
-        os.mkdir(f"{datasets_root}data/")
-    data_path = f"{datasets_root}data/{dataset_name}/"
+        os.mkdir(adapt_path(f"{datasets_root}data/"))
+    data_path = adapt_path(f"{datasets_root}data/{dataset_name}/")
     if not path.exists(data_path):
         logger.log(
             logging.DEBUG,
@@ -117,7 +120,7 @@ def download_dataset(dataset_name: str) -> str:
 
     # Loading dataset
     dl_url = url_to_dllink(dataset["zip_link"])
-    zip_path = f"{data_path}{dataset['zip_name']}"
+    zip_path = adapt_path(f"{data_path}{dataset['zip_name']}")
     try:
         urllib.request.urlretrieve(dl_url, zip_path, print_download_state)
         logger.log(logging.DEBUG, "")
