@@ -2,7 +2,6 @@
 
 from .algorithms.lisi import LISI
 from .algorithms.neighborentropy import NeighborEntropy
-from .algorithms.neighborconservation import NeighborConservation
 from .checking import Checking
 
-__all__ = ["Checking", "LISI", "NeighborConservation", "NeighborEntropy"]
+__all__ = ["Checking", "LISI", "NeighborEntropy"]
