@@ -11,11 +11,10 @@ from .containstransformations import ContainsTransformations
 from .hasmetadata import HasMetadata
 from .isprofilable import IsProfilable, profile_method
 from .isrepresentable import IsRepresentable
-from .issubsamplable import IsSubsamplable
+from .issubsamplable import UsesSubsampling
 from .usescommonfeatures import UsesCommonFeatures
 from .usesmatching import UsesMatching, _TypeMatchingModes
 from .usesmetric import UsesMetric
-from .usesneighbors import UsesNeighbors
 from .usesreference import UsesReference
 from .usessamplelabels import UsesSampleLabels
 from .utils import assert_trait
@@ -28,12 +27,11 @@ __all__ = [
     "IsProfilable",
     "profile_method",
     "IsRepresentable",
-    "IsSubsamplable",
+    "UsesSubsampling",
     "UsesCommonFeatures",
     "UsesMatching",
     "_TypeMatchingModes",
     "UsesMetric",
-    "UsesNeighbors",
     "UsesReference",
     "UsesSampleLabels",
     "assert_trait",
