@@ -1,8 +1,10 @@
 #!/usr/bin/env python3
 
+import anndata as ad
 import igraph as ig
 import leidenalg as la
 import numpy as np
+import scanpy as sc
 import warnings
 
 from anndata import AnnData
@@ -12,31 +14,47 @@ from pynndescent import NNDescent
 from scipy.spatial.distance import cdist
 from scipy.sparse import csr_matrix
 from scipy.sparse.csgraph import dijkstra
-from sklearn.neighbors import NearestNeighbors
+from sklearn import neighbors as skn
 from typing import Dict, List, Literal, Optional, Tuple
 
 from .anndata_manager import anndata_manager as adm
-from .dimred import pca
 from .geometry import sparse_cdist
 from .matrix import (
     extract_chunks,
-    contains_duplicates,
-    perturbate,
     sort_sparse_matrix,
     sparse_from_arrays,
 )
-from .._logging import logger
+
+
+def nearest_neighbors_custom(
+    X: np.ndarray,
+    mode: Literal["edges", "distances"],
+    n_neighbors: int = 15,
+    Y: Optional[np.ndarray] = None,
+) -> csr_matrix:
+    """
+    Wraps sklearn.neighbors.NearestNeighbors class.
+    """
+    if Y is None:
+        Y = X
+    nn_model = skn.NearestNeighbors(n_neighbors=n_neighbors).fit(Y)
+    if mode == "edges":
+        return nn_model.kneighbors_graph(X, mode="connectivity")
+    elif mode == "distances":
+        return nn_model.kneighbors_graph(X, mode="distance").toarray()
+    else:
+        raise ValueError(f"Unrecognized mode: {mode}")
 
 
 def nearest_neighbors(
-    X: np.ndarray,
-    mode: Literal["distances", "edges"] = "distances",
-    algorithm: Literal["auto", "sklearn", "nndescent"] = "auto",
-    n_neighbors: Optional[int] = None,
-    metric: Optional[str] = None,
-    metric_kwargs: Optional[Dict] = None,
-    use_pcs: Optional[int] = None,
-    random_seed: Optional[int] = None,
+    adata: ad.AnnData,
+    mode: Literal["edges", "distances", "connectivities"],
+    neighbors_key: str = "neighbors",
+    n_neighbors: int = 15,
+    metric: str = "euclidean",
+    metric_kwargs: Dict = {},
+    use_pca: bool = True,
+    random_state: Optional[RandomState] = None,
 ) -> csr_matrix:
     """
     Encapsulates k-nearest neighbors algorithms.
@@ -67,91 +85,37 @@ def nearest_neighbors(
         for small datasets or if the solution must be exact, use "nndescent"
         for large datasets if an approached solution is enough. With "auto",
         the function will adapt to dataset size.
-
     """
-    # Retrieves default parameters if needed
-    from .. import settings, use_setting
+    assert isinstance(
+        adata, ad.AnnData
+    ), f"AnnData expected, found {type(adata).__name__}."
+    assert mode in ["edges", "distances", "connectivities"]
+    if (
+        neighbors_key not in adata.uns
+        or "params" not in adata.uns[neighbors_key]
+        or "n_neighbors" not in adata.uns[neighbors_key]["params"]
+        or "metric" not in adata.uns[neighbors_key]["params"]
+        or adata.uns[neighbors_key]["params"]["n_neighbors"] < n_neighbors
+        or adata.uns[neighbors_key]["params"]["metric"] != metric
+    ):
+        from .._settings import settings, use_setting
 
-    n_neighbors = use_setting(n_neighbors, settings.n_neighbors_max)
-    metric = use_setting(metric, settings.neighbors_metric)
-    metric_kwargs = use_setting(metric_kwargs, settings.neighbors_metric_kwargs)
-    use_pcs = use_setting(use_pcs, settings.neighbors_n_pcs)
-    random_seed = use_setting(random_seed, settings.neighbors_random_seed)
-
-    # Checks parameters
-    nx = X.shape[0]
-    if algorithm == "nndescent":
-        use_nndescent = True
-    elif algorithm == "sklearn":
-        use_nndescent = False
-    elif algorithm == "auto":
-        use_nndescent = nx > settings.large_dataset_threshold
-    else:
-        raise ValueError(
-            f"Unrecognized algorithm: {algorithm}. Valid options are 'auto',"
-            " 'nndescent', 'sklearn'."
-        )
-    assert mode in ("edges", "distances"), f"Unknown mode: {mode}."
-    assert use_pcs is None or use_pcs > 0, f"Invalid PC number: {use_pcs}"
-
-    if use_pcs is not None and use_pcs < X.shape[1]:
-        logger.debug(f"nearest_neighbors > Computing PCA {X.shape[1]} -> {use_pcs}")
-        X = pca(X, n_components=use_pcs)
-
-    # If overlapping points, adds a light noise to guarantee
-    # NN algorithms proper functioning.
-    if contains_duplicates(X):
-        logger.debug("nearest_neighbors > Duplicates detected. Jittering.")
-        X = perturbate(X, std=0.01)
-
-    # By default, self loops are included in algorithms
-    n_neighbors += 1
-    n_neighbors = min(n_neighbors, nx - 1)
-
-    # Nearest neighbors
-    connectivity = None
-    if use_nndescent:
-        # PyNNDescent provides a high speed implementation of kNN
-        # Parameters borrowed from UMAP's implementation
-        # https://github.com/lmcinnes/umap
-        logger.debug("nearest_neighbors > Computing nearest neighbors using nndescent.")
-        q_tree = NNDescent(
-            X,
+        if use_pca and "X_pca" not in adata.obsm:
+            sc.pp.pca(adata)
+        sc.pp.neighbors(
+            adata,
             n_neighbors=n_neighbors,
             metric=metric,
             metric_kwds=metric_kwargs,
-            random_state=RandomState(random_seed),
+            method="umap",
+            random_state=use_setting(random_state, settings.global_random_seed),
         )
-        knn_indices, knn_distances = q_tree.neighbor_graph
-        knn_indices = knn_indices[:, 1:]
-        knn_distances = knn_distances[:, 1:]
-        if mode == "distances":
-            connectivity = sparse_from_arrays(knn_indices, knn_distances)
-        else:
-            connectivity = sparse_from_arrays(knn_indices)
-    else:
-        # Standard exact kNN using sklearn implementation
-        logger.debug("nearest_neighbors > Computing nearest neighbors using sklearn.")
-
-        nn = NearestNeighbors(
-            n_neighbors=n_neighbors,
-            metric=metric,
-            metric_params=metric_kwargs,
-        )
-        nn.fit(X)
-        if mode == "distances":
-            nnmode = "distance"
-        else:
-            nnmode = "connectivity"
-        connectivity = nn.kneighbors_graph(X, mode=nnmode)
-
-        connectivity.setdiag(0)
-        connectivity.eliminate_zeros()
-
-    ks = (connectivity > 0).sum(axis=1)
-    kmin, kmax = ks.min(), ks.max()
-    logger.debug(f"nearest_neighbors > n: {X.shape[0]}, kmin: {kmin}, kmax: {kmax}")
-    return connectivity
+    if mode == "edges":
+        return adata.obsp["distances"].astype(bool)
+    elif mode == "distances":
+        return adata.obsp["distances"]
+    elif mode == "connectivities":
+        return adata.obsp["connectivities"]
 
 
 def distance_to_knn(D: np.ndarray, k: int, axis: int):
@@ -312,75 +276,6 @@ def raw_mutual_nearest_neighbors(
     return csr_matrix((D <= Dxy), shape=D.shape, dtype=np.float32)
 
 
-def vertex_cover(adjacency: csr_matrix, hops: int = 1) -> Tuple[np.ndarray, np.ndarray]:
-    """
-    Computes a vertex cover S of a graph G = (V, E) defined by its adjacency
-    matrix. S has a fundamental property: for all v in V, either v in S or
-    there exists (v, x) in E so that x in V. S is a subset of V that can
-    then be used as a subsampling for heavy computational tasks. The heuristic
-    we implemented is a O(n) in time, and returns a vertex cover at most twice
-    as large as the smallest one.
-
-    Parameters
-    ----------
-    adjacency: csr_matrix
-        Adjacency matrix of the graph to subsample, treated as symmetrical.
-
-    hops: int, default = 1
-        Maximal distance to a selected node to be considered covered.
-
-    Returns
-    -------
-    (cover, mapping): Tuple[np.ndarray, np.ndarray]
-        cover: Boolean vector representing belonging to the cover
-        mapping: mapping[i] is the index of the node covering node i.
-    """
-    assert hops >= 0, "Hops must be non-negative."
-    if hops == 0:
-        n = adjacency.shape[0]
-        cover, mapping = np.ones(n), np.arange(n)
-    else:  # use a numba-accelerated function
-        cover, mapping = vertex_cover_njit(
-            adjacency.indptr,
-            adjacency.indices,
-            hops=hops,
-        )
-    return cover.astype(bool), mapping.astype(int)
-
-
-@njit
-def vertex_cover_njit(
-    ptr: np.ndarray,
-    ind: np.ndarray,
-    hops: int = 1,
-) -> Tuple[np.ndarray, np.ndarray]:
-    # ptr, ind: CSR matrix representation
-    # of an adjacency matrix
-    n = ptr.shape[0] - 1
-    anchors = np.zeros(n, dtype="int")
-    for v in range(n):
-        # If v not visited, add it as an anchor
-        if anchors[v]:
-            continue
-        anchors[v] = v
-        # Mark its neighbors as visited
-        neighbors = [(v, 0)]
-        while len(neighbors):
-            nb, d = neighbors.pop()
-            anchors[nb] = v
-            if d < hops:
-                M = ptr[nb + 1] if nb + 1 < n else n
-                for nb2 in ind[ptr[nb] : M]:
-                    if anchors[nb2]:
-                        continue
-                    anchors[nb2] = v
-                    neighbors.append((nb2, d + 1))
-    anchors_set = np.zeros(n)
-    for i in set(anchors):
-        anchors_set[i] = 1
-    return anchors_set, anchors  # set, map
-
-
 def combine_matchings(
     matchings: Dict[Tuple[int, int], csr_matrix],
     knn_graphs: Optional[List[csr_matrix]] = None,
@@ -498,6 +393,8 @@ def _generate_membership_matrix_njit(
     """
     Numba accelerated helper
     """
+    distances = distances.copy()
+
     n1 = distances.shape[0]
     # Binary search
     for row_i in range(n1):
@@ -605,7 +502,7 @@ def cluster_anndatas(
     datasets: List[AnnData],
     use_rep: Optional[str] = None,
     cluster_key: str = "cluster",
-    n_neighbors: int = 10,
+    n_neighbors: int = 15,
     resolution: float = 1.0,
 ) -> None:
     """
@@ -636,10 +533,10 @@ def cluster_anndatas(
         adm.isset_value(adata, key=use_rep, field="obsm") for adata in datasets
     ), f"{use_rep} missing in .obsm of some AnnDatas."
     X = np.concatenate(
-        [adm.get_value(adata, key=use_rep, field_str="obsm") for adata in datasets],
+        [adata.obsm[use_rep] for adata in datasets],
         axis=0,
     )
-    adj_matrix = nearest_neighbors(X, mode="edges", n_neighbors=n_neighbors)
+    adj_matrix = nearest_neighbors_custom(X, mode="edges", n_neighbors=n_neighbors)
     sources, targets = adj_matrix.nonzero()
     edgelist = zip(sources.tolist(), targets.tolist())
     partition = np.array(
