@@ -1,23 +1,30 @@
 #!/usr/bin/env python3
 
+import matplotlib as mt
 import matplotlib.pyplot as plt
+from mpl_toolkits.axes_grid1 import make_axes_locatable
 import numpy as np
-import os
 
 from anndata import AnnData
 from matplotlib import cm
 from numbers import Number
-from os.path import exists
 from scipy.sparse import csr_matrix
 from typing import Dict, List, Literal, Optional, Union
 
+
 from ..engine import Model
 from ..engine.evaluators import (
     evaluate_matching_layer,
     matching_edge_accuracy_discrete,
     matching_edge_penalty_continuous,
 )
-from ..engine.layers import LayerMatching
+from ..engine.layers import (
+    Layer,
+    LayerMatching,
+    LayerMerging,
+)
+from ..engine.matching import CombineMatching
+from ..engine.traits import ContainsTransformations
 from ..utils.anndata_manager import (
     anndata_manager as adm,
     AnnDataKeyIdentifiers,
@@ -26,8 +33,6 @@ from ..utils.anndata_manager import (
 from ..utils.dimred import pca, umap
 from ..utils.matrix import extract_chunks, guess_is_discrete
 
-MARKERS = "osv^<>pP*hHXDd"
-
 
 def reduce_dimension(
     datasets: Union[AnnData, List[AnnData], Dict[str, AnnData]],
@@ -111,30 +116,19 @@ def reduce_dimension(
 
 def scatter_plot(
     datasets: Union[AnnData, List[AnnData], Dict[str, AnnData]],
-    matching_mtx: Optional[csr_matrix] = None,
-    input_obsm: Optional[str] = None,
-    color_by: str = "__dataset__",
-    palette: str = "rainbow",
-    title: Optional[str] = None,
+    use_rep: Optional[str] = None,
+    color_by: Optional[str] = None,
     xlabel: Optional[str] = None,
     ylabel: Optional[str] = None,
-    batch_names: Optional[List[int]] = None,
-    show_title: bool = True,
-    show_legend: bool = True,
-    plot_cluster_names: bool = False,
-    show: bool = True,
-    save: bool = False,
-    extension: str = "png",
-    caller_path: str = "",
-    suffix: str = "__color_by__",
+    title: Optional[str] = None,
+    dataset_labels: Optional[List[str]] = None,
+    labels_on_plot: bool = False,
+    palette: str = "rainbow",
+    dot_size: float = 0.5,
     dpi: int = 100,
 ):
     """
-    Advanced plotting function for transmorph results, handling parameters
-    guessing, representation cacheing, automatic dimensionality reduction,
-    continuous or linear labeling, figure saving.
-
-    IMPORTANT: If save=True, see parameter caller_path.
+    Advanced plotting function for transmorph results.
 
     Parameters
     ----------
@@ -144,64 +138,37 @@ def scatter_plot(
         Otherwise, AnnData.X is chosen as fallback representation (in this
         case, all datasets must be embedded in the same space).
 
-    matching_mtx: csr_matrix, default = None
-        This option is only valid for two datasets. You can provide a boolean
-        or probabilistic matching between datasets represented as a sparse
-        matrix (CSR format), which will be displayed on the plot. Be careful,
-        for too large datasets this will slow down plotting and yield results
-        difficult to read.
+    use_rep: Optional[str]
+        AnnData.obsm key containing data embedding. If None, attempts to use
+        AnnData.X instead if dimensionality is 2.
 
-    reducer: str, default = "umap"
-        Algorithm to use if embedding dimensionality exceeds 2. Valid options
-        are "pca" for linear dimensionality reduction and "umap" for embedding
-        high dimensional datasets such as single-cell data.
-
-    color_by: str, default = "__dataset__"
-        Labeling colors to use in the figure. "__dataset__" will color each
+    color_by: Optional[str]
+        Labeling colors to use in the figure. None will color each
         point depending of its dataset of origin. Otherwise, all keys of
         AnnData.obs are valid. The function will try to guess discrete- or
         continuous-ness of the labels, and color accordingly.
 
+    xlabel: Optional[str]
+        Labeling of the x axis.
+
+    ylabel: Optional[str]
+        Labeling of the y axis.
+
+    title: Optional[str]
+        Title of the plot.
+
+    dataset_labels: Optional[List[str]]
+        Labels to use in legend to describe each AnnData, if datasets
+        is not a Dict.
+
+    labels_on_plot: bool, default = False
+        Shows labels on plot instead of in legend.
+
     palette: str, default = "rainbow"
         Matplotlib colormap to pick colors from.
 
-    title: str, default = ""
-        Title of the plot.
-
-    xlabel: str, default = ""
-        Labeling of the x axis.
-
-    ylabel: str, default = ""
-        Labeling of the y axis.
-
-    show_title: bool, default = True
-        Shows plot title.
-
-    show_legend: bool, default = True
-        Shows the legend panel.
-
-    plot_cluster_names: bool, default = False
-        Shows cluster names on the plot, gathered from color_by.
-
-    show: bool, default = True
-        Call plt.show() at the end to display the figure in a separate
-        window.
-
-    save: bool, default = False
-        Save the plot in the same directory as the file calling the
-        functions, in a figures/ subdirectory (created if necessary).
-
-    extension: str, default = "png"
-        If save=True, valid image file extension to use.
-
-    caller_path:
-        If save=True, pass f"__file__" to indicate file location.
-
-    suffix: str = "__color_by__":
-        If save=True, suffix to append to the file name to differentiate
-        between image files of the same source. "__color_by__" will use
-        labels as suffix (e.g. "plot_cell_type.png")
-                                     ^^^^^^^^^
+    dot_size: float, default = 0.5
+        Scatter "size" parameter.
 
     dpi: int, default = 200,
         Dot per inch to use for the figure.
@@ -209,35 +176,31 @@ def scatter_plot(
     if isinstance(datasets, AnnData):
         datasets = [datasets]
     if isinstance(datasets, Dict):
-        batch_names = list(datasets.keys())
+        dataset_labels = list(datasets.keys())
         datasets = list(datasets.values())
 
     # Checking parameters
     assert all(
         isinstance(adata, AnnData) for adata in datasets
     ), "All datasets must be AnnData."
-    if matching_mtx is not None:
-        assert len(datasets) == 2, "Drawing matching requires exactly two datasets."
-        assert type(matching_mtx) is csr_matrix, "Matching must come as a csr_matrix."
 
-    if input_obsm is None:
-        input_obsm = AnnDataKeyIdentifiers.PlotRepresentation
+    if use_rep is None:
+        assert all(
+            adata.X.shape[1] == 2 for adata in datasets
+        ), "No use_rep specified, and some adata.X are not 2 dimensional."
 
     # Retrieving representation
     default_xlabel = "Feature 1"
     default_ylabel = "Feature 2"
-    if not all(adm.isset_value(adata, key=input_obsm) for adata in datasets):
-        assert all(
-            adata.X.shape[1] == 2 for adata in datasets
-        ), "Make sure to call reduce_dimension first."
+    if not all(adm.isset_value(adata, key=use_rep) for adata in datasets):
         representations = [adata.X for adata in datasets]
     else:
-        representations = [adm.get_value(adata, key=input_obsm) for adata in datasets]
-        reducer = adm.get_value(datasets[0], key=f"reducer_{input_obsm}")
+        representations = [adm.get_value(adata, key=use_rep) for adata in datasets]
+        reducer = adm.get_value(datasets[0], key=f"reducer_{use_rep}")
         if reducer == "umap":
             default_xlabel = "UMAP1"
             default_ylabel = "UMAP2"
-        else:
+        elif reducer == "pca":
             default_xlabel = "PC1"
             default_ylabel = "PC2"
 
@@ -246,76 +209,50 @@ def scatter_plot(
     if ylabel is None:
         ylabel = default_ylabel
 
-    # Guessing continous or discrete labels
-    if color_by == "__dataset__":
-        n_labels = len(datasets)
-        continuous = False
-        if batch_names is not None:
-            all_labels = batch_names
+    if color_by is None:  # Color by dataset
+        continuous_palette = False
+        if dataset_labels is not None:
+            all_labels = dataset_labels
         else:
-            all_labels = [f"Batch {i + 1}" for i in range(n_labels)]
-    else:
+            all_labels = [f"Dataset {i + 1}" for i, _ in enumerate(datasets)]
+    else:  # Colour by custom label
         all_labels = set()
         for adata in datasets:
             all_labels = all_labels | set(adata.obs[color_by])
         all_labels = sorted(all_labels)
-        n_labels = len(all_labels)
-        continuous = (n_labels > 20) and all(isinstance(y, Number) for y in all_labels)
+        # simple heuristic
+        continuous_palette = (len(all_labels) > 20) and all(
+            isinstance(y, Number) for y in all_labels
+        )
 
-    # Guess plotting parameters
-    npoints = sum(X.shape[0] for X in representations)
-    size = 100
-    if npoints > 100:
-        size = 60
-    if npoints > 500:
-        size = 50
-    if npoints > 1000:
-        size = 40
-    if npoints > 5000:
-        size = 30
-    if npoints > 10000:
-        size = 20
-    if npoints > 50000:
-        size = 10
-
-    ndatasets = len(datasets)
-    alpha = 0.8
-    if ndatasets >= 3:
-        alpha = 0.6
-    if ndatasets >= 10:
-        alpha = 0.4
+    dot_alpha = 1
+    n_labels = len(all_labels)
+    n_datasets = len(datasets)
 
     # Prepare the palette
     cmap = cm.get_cmap(palette, n_labels)
 
     # Do the plotting
-    plt.figure(figsize=(6, 6), dpi=dpi)
+    fig = plt.figure(dpi=dpi)
+    ax_scatter = fig.add_subplot(111, aspect="equal")
+
+    scatter = None
     for i, adata in enumerate(datasets):
         X = representations[i]
-        mk = MARKERS[i % len(MARKERS)]  # Loops to avoid oob error
-        if color_by == "__dataset__":
-            if ndatasets == 1:
+        if color_by is None:
+            if n_datasets == 1:
                 color = cmap(0.5)
             else:
-                color = cmap(i / (ndatasets - 1))
-            plt.scatter(*X.T, marker=mk, s=size, alpha=alpha, color=color)
-            plt.scatter(
-                [],
-                [],
-                marker=mk,
-                s=40,
-                label=all_labels[i],
-                color=color,
+                color = cmap(i / (n_datasets - 1))
+            scatter = ax_scatter.scatter(
+                *X.T, s=dot_size, alpha=dot_alpha, color=color, label=all_labels[i]
             )
-            continue
-        if continuous:
-            show_legend = False
-            plt.scatter(
+        elif continuous_palette:
+            scatter = ax_scatter.scatter(
                 *X.T,
                 c=adata.obs[color_by],
-                alpha=alpha,
-                marker=mk,
-                s=size,
+                alpha=dot_alpha,
+                s=dot_size,
                 cmap=palette,
             )
         else:
@@ -325,26 +262,26 @@ def scatter_plot(
                 else:
                     color = cmap(k / (n_labels - 1))
                 if i == 0:
-                    plt.scatter(
-                        [],
-                        [],
-                        marker="o",
-                        s=40,
+                    scatter = ax_scatter.scatter(
+                        *X[adata.obs[color_by] == label].T,
+                        alpha=dot_alpha,
+                        s=dot_size,
                         color=color,
                         label=label,
                     )
-                plt.scatter(
-                    *X[adata.obs[color_by] == label].T,
-                    alpha=alpha,
-                    marker=mk,
-                    s=size,
-                    color=color,
-                )
-    if continuous:
-        plt.colorbar()
+                else:
+                    scatter = ax_scatter.scatter(
+                        *X[adata.obs[color_by] == label].T,
+                        alpha=dot_alpha,
+                        s=dot_size,
+                        color=color,
+                    )
 
-    # Plotting cluster names if necessary
-    if plot_cluster_names:
+    if continuous_palette:
+        divider = make_axes_locatable(ax_scatter)
+        ax_legend = divider.append_axes("right", size="5%", pad=0.1)
+        plt.colorbar(scatter, cax=ax_legend).set_label(color_by, fontsize=14)
+    elif labels_on_plot:
         for ilabel, label in enumerate(all_labels):
             nobs, cl_pos = 0, np.zeros((2,), dtype=np.float32)
             for i, adata in enumerate(datasets):
@@ -366,64 +303,34 @@ def scatter_plot(
                 ha="center",
                 va="center",
             )
-
-    # Drawing matching if necessary
-    if matching_mtx is not None:
-        Tcoo = matching_mtx.tocoo()
-        X1, X2 = representations
-        for i, j, v in zip(Tcoo.row, Tcoo.col, Tcoo.data):
-            plt.plot(
-                [X1[i][0], X2[j][0]],
-                [X1[i][1], X2[j][1]],
-                alpha=np.clip(v * X2.shape[0], 0, 1),
-                c="k",
-            )
-
-    # Reordering legend
-    if show_legend:
-        handles, labels = plt.gca().get_legend_handles_labels()
-        if len(handles) < 5:
-            legendsize = 12
-        elif len(handles) < 10:
-            legendsize = 8
-        else:
-            legendsize = 5
+    else:
+        divider = make_axes_locatable(ax_scatter)
+        ax_legend = divider.append_axes("right", size="5%", pad=0.1)
+        handles, labels = ax_scatter.get_legend_handles_labels()
         order = np.argsort(labels)
-        plt.legend(
+        for label in order:
+            ax_legend.scatter([], [], label=label)
+        legend = ax_legend.legend(
             [handles[idx] for idx in order],
             [labels[idx] for idx in order],
-            fontsize=legendsize,
+            fontsize=12,
+            ncols=(1 + len(order) // 10),
+            loc="center left",
         )
+        legend.legendHandles[0]._sizes = [15]
+        legend.legendHandles[1]._sizes = [15]
+        ax_legend.axis("off")
 
     # Adding text pieces
-    plt.xlabel(xlabel, fontsize=16)
-    plt.ylabel(ylabel, fontsize=16)
-    plt.xticks([])
-    plt.yticks([])
-    if color_by == "__dataset__":
-        color_by = "dataset"
-    if show_title:
-        if title is None:
-            title = ""
-        plt.title(title, fontsize=18)
+    ax_scatter.set_xlabel(xlabel, fontsize=16)
+    ax_scatter.set_ylabel(ylabel, fontsize=16)
+    ax_scatter.set_xticks([])
+    ax_scatter.set_yticks([])
 
-    # Saving, showing and closing
-    if save:
-        if "\\" in caller_path:
-            sep_char = "\\"
-        else:
-            sep_char = "/"
-        save_path = sep_char.join(caller_path.split(sep_char)[:-1]) + f"{sep_char}figures{sep_char}"
-        if not exists(save_path):
-            os.mkdir(save_path)
-        fname = caller_path.split(sep_char)[-1].split(".")[0]
-        if suffix == "__color_by__":
-            suffix = color_by
-        if suffix != "":
-            suffix = "_" + suffix
-        plt.savefig(save_path + f"{fname}{suffix}.{extension}")
-    if show:
-        plt.show()
+    if title is not None:
+        ax_scatter.set_title(title, fontsize=18)
+
+    plt.show()
     plt.close()
 
 
@@ -455,6 +362,8 @@ def plot_matching_eval(
     matshow_kwargs: Dict[str, Any], default = {}
         Additional matshow parameters.
     """
+    if isinstance(datasets, AnnData):
+        datasets = {"Batch": datasets}
     if isinstance(datasets, Dict):
         dataset_keys = list(datasets.keys())
         datasets = list(datasets.values())
@@ -555,3 +464,112 @@ def plot_label_distribution_heatmap(
         plt.yticks(np.arange(ndatasets), dataset_keys)
     if title is not None:
         plt.title(title)
+
+
+def plot_model(
+    model: Model,
+    layer_edgecolor: str = "royalblue",
+    layer_facecolor: str = "lightsteelblue",
+    algorithm_edgecolor: str = "darkorange",
+    algorithm_facecolor: str = "bisque",
+) -> None:
+    """
+    Plots a visual representation of a model.
+    """
+
+    BOX_WIDTH = 40
+    BOX_HEIGHT = 20
+
+    def _retrieve_elements(layer: Layer):
+        elements = []
+        if isinstance(layer, ContainsTransformations):
+            for transformation in layer.transformations:
+                elements.append(type(transformation).__name__)
+        if isinstance(layer, LayerMatching):
+            elements.append(type(layer.matching).__name__)
+            if isinstance(layer.matching, CombineMatching):
+                for matching in layer.matching.matchings:
+                    elements.append(type(matching).__name__)
+        if isinstance(layer, LayerMerging):
+            elements.append(type(layer.merging).__name__)
+        return elements
+
+    def _retrieve_n_rows(model: Model) -> int:
+        max_elements = 0
+        for layer in model.layers:
+            max_elements = max(max_elements, len(_retrieve_elements(layer)))
+        return max_elements
+
+    def _plot_textbox(x: int, y: int, text: str, ax: mt.axes.Axes, **rect_kwargs):
+        rect = mt.patches.Rectangle((x, y), BOX_WIDTH, BOX_HEIGHT, **rect_kwargs)
+        ax.add_patch(rect)
+        ax.text(
+            x + BOX_WIDTH / 2,
+            y + BOX_HEIGHT / 2,
+            text,
+            ha="center",
+            va="center",
+            fontsize=9,
+        )
+
+    def _plot_layer(layer: Layer, x_offset: int, ax: mt.axes.Axes):
+        _plot_textbox(
+            x_offset,
+            0,
+            type(layer).__name__[5:],
+            ax,
+            linewidth=1,
+            edgecolor=layer_edgecolor,
+            facecolor=layer_facecolor,
+        )
+        next_layers = layer.output_layers
+        if len(next_layers) > 1:
+            raise NotImplementedError
+        if len(next_layers) == 0:
+            return
+        plt.arrow(
+            x_offset + BOX_WIDTH,
+            BOX_HEIGHT / 2,
+            BOX_WIDTH / 2,
+            0,
+            length_includes_head=True,
+            facecolor="k",
+            head_width=3,
+            head_length=1.5,
+        )
+        y_offset = 0
+        for element in _retrieve_elements(layer):
+            plt.arrow(
+                x_offset + BOX_HEIGHT,
+                y_offset,
+                0,
+                -BOX_HEIGHT,
+                length_includes_head=True,
+                facecolor="k",
+                head_width=2,
+            )
+            _plot_textbox(
+                x_offset,
+                y_offset - BOX_HEIGHT * 2,
+                element,
+                ax,
+                linewidth=1,
+                edgecolor=algorithm_edgecolor,
+                facecolor=algorithm_facecolor,
+            )
+            y_offset -= BOX_HEIGHT * 2
+        _plot_layer(next_layers[0], x_offset + BOX_WIDTH * 1.5, ax)
+
+    assert isinstance(model, Model), f"Model expected, found {type(model)}."
+
+    n_layers = len(model.layers)
+    n_rows = _retrieve_n_rows(model)
+
+    if n_layers == 0:
+        return
+
+    fig = plt.figure(figsize=(2.5 * n_layers, 0.7 * (1 + n_rows)))
+    ax = fig.add_subplot(111)
+    plt.axis("off")
+
+    _plot_layer(model.layers[0], 0, ax)
